@@ -161,17 +161,8 @@ static const char *control_access(snd_ctl_elem_info_t *info)
 	return result;
 }
 
-static int check_range(int val, int min, int max)
-{
-	if (no_check)
-		return val;
-	if (val < min)
-		return min;
-	if (val > max)
-		return max;
-	return val;
-}
-
+#define check_range(val, min, max) \
+	(no_check ? (val) : ((val < min) ? (min) : (val > max) ? (max) : (val))) 
 #if 0
 static int convert_range(int val, int omin, int omax, int nmin, int nmax)
 {
@@ -210,17 +201,8 @@ static int convert_prange(int val, int min, int max)
 
 /* Function to convert from percentage to volume. val = percentage */
 
-static int convert_prange1(int val, int min, int max)
-{
-	int range = max - min;
-	int tmp;
-
-	if (range == 0)
-		return 0;
-
-	tmp = rint((double)range * ((double)val*.01)) + min;
-	return tmp;
-}
+#define convert_prange1(val, min, max) \
+	ceil((val) * ((max) - (min)) * 0.01 + (min))
 
 static const char *get_percent(int val, int min, int max)
 {
@@ -247,90 +229,167 @@ static const char *get_percent1(int val, int min, int max, int min_dB, int max_d
 
 static long get_integer(char **ptr, long min, long max)
 {
-	int tmp, tmp1, tmp2;
+	long val = min;
+	char *p = *ptr, *s;
 
-	if (**ptr == ':')
-		(*ptr)++;
-	if (**ptr == '\0' || (!isdigit(**ptr) && **ptr != '-'))
-		return min;
-	tmp = strtol(*ptr, ptr, 10);
-	tmp1 = tmp;
-	tmp2 = 0;
-	if (**ptr == '.') {
-		(*ptr)++;
-		tmp2 = strtol(*ptr, ptr, 10);
+	if (*p == ':')
+		p++;
+	if (*p == '\0' || (!isdigit(*p) && *p != '-'))
+		goto out;
+
+	s = p;
+	val = strtol(s, &p, 10);
+	if (*p == '.') {
+		p++;
+		strtol(p, &p, 10);
 	}
-	if (**ptr == '%') {
-		tmp1 = convert_prange1(tmp, min, max);
-		(*ptr)++;
+	if (*p == '%') {
+		val = (long)convert_prange1(strtod(s, NULL), min, max);
+		p++;
 	}
-	tmp1 = check_range(tmp1, min, max);
-	if (**ptr == ',')
-		(*ptr)++;
-	return tmp1;
+	val = check_range(val, min, max);
+	if (*p == ',')
+		p++;
+ out:
+	*ptr = p;
+	return val;
 }
 
 static long get_integer64(char **ptr, long long min, long long max)
 {
-	long long tmp, tmp1, tmp2;
+	long long val = min;
+	char *p = *ptr, *s;
 
-	if (**ptr == ':')
-		(*ptr)++;
-	if (**ptr == '\0' || (!isdigit(**ptr) && **ptr != '-'))
-		return min;
-	tmp = strtol(*ptr, ptr, 10);
-	tmp1 = tmp;
-	tmp2 = 0;
-	if (**ptr == '.') {
-		(*ptr)++;
-		tmp2 = strtol(*ptr, ptr, 10);
+	if (*p == ':')
+		p++;
+	if (*p == '\0' || (!isdigit(*p) && *p != '-'))
+		goto out;
+
+	s = p;
+	val = strtol(s, &p, 10);
+	if (*p == '.') {
+		p++;
+		strtol(p, &p, 10);
 	}
-	if (**ptr == '%') {
-		tmp1 = convert_prange1(tmp, min, max);
-		(*ptr)++;
+	if (*p == '%') {
+		val = (long long)convert_prange1(strtod(s, NULL), min, max);
+		p++;
 	}
-	tmp1 = check_range(tmp1, min, max);
-	if (**ptr == ',')
-		(*ptr)++;
-	return tmp1;
+	val = check_range(val, min, max);
+	if (*p == ',')
+		p++;
+ out:
+	*ptr = p;
+	return val;
 }
 
-static int get_volume_simple(char **ptr, int min, int max, int orig)
-{
-	int tmp, tmp1, tmp2;
+struct volume_ops {
+	int (*get_range)(snd_mixer_elem_t *elem, long *min, long *max);
+	int (*get)(snd_mixer_elem_t *elem, snd_mixer_selem_channel_id_t c,
+		   long *value);
+	int (*set)(snd_mixer_elem_t *elem, snd_mixer_selem_channel_id_t c,
+		   long value);
+};
+	
+enum { VOL_RAW, VOL_DB };
 
-	if (**ptr == ':')
-		(*ptr)++;
-	if (**ptr == '\0' || (!isdigit(**ptr) && **ptr != '-'))
-		return min;
-	tmp = atoi(*ptr);
-	if (**ptr == '-')
-		(*ptr)++;
-	while (isdigit(**ptr))
-		(*ptr)++;
-	tmp1 = tmp;
-	tmp2 = 0;
-	if (**ptr == '.') {
-		(*ptr)++;
-		tmp2 = atoi(*ptr);
-		while (isdigit(**ptr))
-			(*ptr)++;
+struct volume_ops_set {
+	int (*has_volume)(snd_mixer_elem_t *elem);
+	struct volume_ops v[2];
+};
+
+static int set_playback_dB(snd_mixer_elem_t *elem,
+			   snd_mixer_selem_channel_id_t c, long value)
+{
+	return snd_mixer_selem_set_playback_dB(elem, c, value, 0);
+}
+
+static int set_capture_dB(snd_mixer_elem_t *elem,
+			  snd_mixer_selem_channel_id_t c, long value)
+{
+	return snd_mixer_selem_set_capture_dB(elem, c, value, 0);
+}
+
+static struct volume_ops_set vol_ops[2] = {
+	{
+		.has_volume = snd_mixer_selem_has_playback_volume,
+		.v = {{ snd_mixer_selem_get_playback_volume_range,
+			snd_mixer_selem_get_playback_volume,
+			snd_mixer_selem_set_playback_volume },
+		      { snd_mixer_selem_get_playback_dB_range,
+			snd_mixer_selem_get_playback_dB,
+			set_playback_dB }},
+	},
+	{
+		.has_volume = snd_mixer_selem_has_capture_volume,
+		.v = {{ snd_mixer_selem_get_capture_volume_range,
+			snd_mixer_selem_get_capture_volume,
+			snd_mixer_selem_set_capture_volume },
+		      { snd_mixer_selem_get_capture_dB_range,
+			snd_mixer_selem_get_capture_dB,
+			set_capture_dB }},
+	},
+};
+
+static int set_volume_simple(snd_mixer_elem_t *elem,
+			     snd_mixer_selem_channel_id_t chn,
+			     char **ptr, int dir)
+{
+	long val, orig, pmin, pmax;
+	char *p = *ptr, *s;
+	int invalid = 0, err = 0, vol_type = VOL_RAW;
+
+	if (! vol_ops[dir].has_volume(elem))
+		invalid = 1;
+
+	if (*p == ':')
+		p++;
+	if (*p == '\0' || (!isdigit(*p) && *p != '-'))
+		goto skip;
+
+	if (! invalid &&
+	    vol_ops[dir].v[VOL_RAW].get_range(elem, &pmin, &pmax) < 0)
+		invalid = 1;
+
+	s = p;
+	val = strtol(s, &p, 10);
+	if (*p == '.') {
+		p++;
+		strtol(p, &p, 10);
 	}
-	if (**ptr == '%') {
-		tmp1 = convert_prange1(tmp, min, max);
-		(*ptr)++;
+	if (*p == '%') {
+		if (! invalid)
+			val = (long)convert_prange1(strtod(s, NULL), pmin, pmax);
+		p++;
+	} else if (p[0] == 'd' && p[1] == 'B') {
+		if (! invalid) {
+			val = (long)(strtod(s, NULL) * 100.0);
+			vol_type = VOL_DB;
+			if (vol_ops[dir].v[vol_type].get_range(elem, &pmin, &pmax) < 0)
+				invalid = 1;
+		}
+		p += 2;
 	}
-	if (**ptr == '+') {
-		tmp1 = orig + tmp1;
-		(*ptr)++;
-	} else if (**ptr == '-') {
-		tmp1 = orig - tmp1;
-		(*ptr)++;
+	if (*p == '+' || *p == '-') {
+		if (! invalid) {
+			if (vol_ops[dir].v[vol_type].get(elem, chn, &orig) < 0)
+				invalid = 1;
+			if (*p == '+')
+				val = orig + val;
+			else
+				val = orig - val;
+		}
+		p++;
 	}
-	tmp1 = check_range(tmp1, min, max);
-	if (**ptr == ',')
-		(*ptr)++;
-	return tmp1;
+	if (! invalid) {
+		val = check_range(val, pmin, pmax);
+		err = vol_ops[dir].v[vol_type].set(elem, chn, val);
+	}
+ skip:
+	if (*p == ',')
+		p++;
+	*ptr = p;
+	return err ? err : (invalid ? -ENOENT : 0);
 }
 
 static int get_bool_simple(char **ptr, char *str, int invert, int orig)
@@ -439,6 +498,22 @@ static void decode_tlv(unsigned int spaces, unsigned int *tlv, unsigned int tlv_
 			printf(",mute=%i", (tlv[3] >> 16) & 1);
 		}
 		break;
+#ifdef SND_CTL_TLVT_DB_LINEAR
+	case SND_CTL_TLVT_DB_LINEAR:
+		printf("dBlinear-");
+		if (size != 2 * sizeof(unsigned int)) {
+			while (size > 0) {
+				printf("0x%x", tlv[idx++]);
+				size -= sizeof(unsigned int);
+			}
+		} else {
+			printf("min=");
+			print_dB(tlv[2]);
+			printf(",max=");
+			print_dB(tlv[3]);
+		}
+		break;
+#endif
 	default:
 		printf("unk-%i-", type);
 		while (size > 0) {
@@ -459,6 +534,7 @@ static int show_control(const char *space, snd_hctl_elem_t *elem,
 	snd_ctl_elem_id_t *id;
 	snd_ctl_elem_info_t *info;
 	snd_ctl_elem_value_t *control;
+	snd_aes_iec958_t iec958;
 	snd_ctl_elem_id_alloca(&id);
 	snd_ctl_elem_info_alloca(&info);
 	snd_ctl_elem_value_alloca(&control);
@@ -530,6 +606,12 @@ static int show_control(const char *space, snd_hctl_elem_t *elem,
 			case SND_CTL_ELEM_TYPE_BYTES:
 				printf("0x%02x", snd_ctl_elem_value_get_byte(control, idx));
 				break;
+			case SND_CTL_ELEM_TYPE_IEC958:
+				snd_ctl_elem_value_get_iec958(control, &iec958);
+				printf("[AES0=0x%02x AES1=0x%02x AES2=0x%02x AES3=0x%02x]",
+				       iec958.status[0], iec958.status[1],
+				       iec958.status[2], iec958.status[3]);
+				break;
 			default:
 				printf("?");
 				break;
@@ -639,7 +721,11 @@ static int show_selem(snd_mixer_t *handle, snd_mixer_selem_id_t *id, const char
 					printf(" cswitch-exclusive");
 			}
 		}
-		if (snd_mixer_selem_is_enumerated(elem)) {
+		if (snd_mixer_selem_is_enum_playback(elem)) {
+			printf(" penum");
+		} else if (snd_mixer_selem_is_enum_capture(elem)) {
+			printf(" cenum");
+		} else if (snd_mixer_selem_is_enumerated(elem)) {
 			printf(" enum");
 		}
 		printf("\n");
@@ -1106,6 +1192,10 @@ static int cset(int argc, char *argv[], int roflag, int keep_handle)
 		if (ignore_error)
 			return 0;
 		error("Cannot find the given element from control %s\n", card);
+		if (! keep_handle) {
+			snd_ctl_close(handle);
+			handle = NULL;
+		}
 		return err;
 	}
 	snd_ctl_elem_info_get_id(info, id);	/* FIXME: Remove it when hctl find works ok !!! */
@@ -1168,10 +1258,13 @@ static int cset(int argc, char *argv[], int roflag, int keep_handle)
 				ptr++;
 		}
 		if ((err = snd_ctl_elem_write(handle, control)) < 0) {
-			if (ignore_error)
-				return 0;
-			error("Control %s element write error: %s\n", card, snd_strerror(err));
-			return err;
+			if (!ignore_error)
+				error("Control %s element write error: %s\n", card, snd_strerror(err));
+			if (!keep_handle) {
+				snd_ctl_close(handle);
+				handle = NULL;
+			}
+			return ignore_error ? 0 : err;
 		}
 	}
 	if (! keep_handle) {
@@ -1278,12 +1371,11 @@ static int get_enum_item_index(snd_mixer_elem_t *elem, char **ptrp)
 
 static int sset(unsigned int argc, char *argv[], int roflag, int keep_handle)
 {
-	int err;
+	int err, check_flag;
 	unsigned int idx;
 	snd_mixer_selem_channel_id_t chn;
 	unsigned int channels = ~0U;
 	unsigned int dir = 3, okflag = 3;
-	long pmin, pmax, cmin, cmax;
 	static snd_mixer_t *handle = NULL;
 	snd_mixer_elem_t *elem;
 	snd_mixer_selem_id_t *sid;
@@ -1337,8 +1429,7 @@ static int sset(unsigned int argc, char *argv[], int roflag, int keep_handle)
 	}
 	if (roflag)
 		goto __skip_write;
-	snd_mixer_selem_get_playback_volume_range(elem, &pmin, &pmax);
-	snd_mixer_selem_get_capture_volume_range(elem, &cmin, &cmax);
+	check_flag = 0;
 	for (idx = 1; idx < argc; idx++) {
 		char *ptr = argv[idx], *optr;
 		int multi, firstchn = 1;
@@ -1353,17 +1444,18 @@ static int sset(unsigned int argc, char *argv[], int roflag, int keep_handle)
 		for (chn = 0; chn <= SND_MIXER_SCHN_LAST; chn++) {
 			char *sptr = NULL;
 			int ival;
-			long lval;
 
 			if (!(channels & (1 << chn)))
 				continue;
 			/* enum control */
 			if (snd_mixer_selem_is_enumerated(elem)) {
 				int idx = get_enum_item_index(elem, &ptr);
-				if (idx < 0)
+				if (idx < 0) {
 					break;
-				else
-					snd_mixer_selem_set_enum_item(elem, chn, idx);
+				} else {
+					if (snd_mixer_selem_set_enum_item(elem, chn, idx) >= 0)
+						check_flag = 1;
+				}
 				if (!multi)
 					ptr = optr;
 				continue;
@@ -1373,29 +1465,30 @@ static int sset(unsigned int argc, char *argv[], int roflag, int keep_handle)
 				sptr = ptr;
 				if (!strncmp(ptr, "mute", 4) && snd_mixer_selem_has_playback_switch(elem)) {
 					snd_mixer_selem_get_playback_switch(elem, chn, &ival);
-					snd_mixer_selem_set_playback_switch(elem, chn, get_bool_simple(&ptr, "mute", 1, ival));
+					if (snd_mixer_selem_set_playback_switch(elem, chn, get_bool_simple(&ptr, "mute", 1, ival)) >= 0)
+						check_flag = 1;
 				} else if (!strncmp(ptr, "off", 3) && snd_mixer_selem_has_playback_switch(elem)) {
 					snd_mixer_selem_get_playback_switch(elem, chn, &ival);
-					snd_mixer_selem_set_playback_switch(elem, chn, get_bool_simple(&ptr, "off", 1, ival));
+					if (snd_mixer_selem_set_playback_switch(elem, chn, get_bool_simple(&ptr, "off", 1, ival)) >= 0)
+						check_flag = 1;
 				} else if (!strncmp(ptr, "unmute", 6) && snd_mixer_selem_has_playback_switch(elem)) {
 					snd_mixer_selem_get_playback_switch(elem, chn, &ival);
-					snd_mixer_selem_set_playback_switch(elem, chn, get_bool_simple(&ptr, "unmute", 0, ival));
+					if (snd_mixer_selem_set_playback_switch(elem, chn, get_bool_simple(&ptr, "unmute", 0, ival)) >= 0)
+						check_flag = 1;
 				} else if (!strncmp(ptr, "on", 2) && snd_mixer_selem_has_playback_switch(elem)) {
 					snd_mixer_selem_get_playback_switch(elem, chn, &ival);
-					snd_mixer_selem_set_playback_switch(elem, chn, get_bool_simple(&ptr, "on", 0, ival));
+					if (snd_mixer_selem_set_playback_switch(elem, chn, get_bool_simple(&ptr, "on", 0, ival)) >= 0)
+						check_flag = 1;
 				} else if (!strncmp(ptr, "toggle", 6) && snd_mixer_selem_has_playback_switch(elem)) {
 					if (firstchn || !snd_mixer_selem_has_playback_switch_joined(elem)) {
 						snd_mixer_selem_get_playback_switch(elem, chn, &ival);
-						snd_mixer_selem_set_playback_switch(elem, chn, (ival ? 1 : 0) ^ 1);
+						if (snd_mixer_selem_set_playback_switch(elem, chn, (ival ? 1 : 0) ^ 1) >= 0)
+							check_flag = 1;
 					}
 					simple_skip_word(&ptr, "toggle");
 				} else if (isdigit(*ptr) || *ptr == '-' || *ptr == '+') {
-					if (snd_mixer_selem_has_playback_volume(elem)) {
-						snd_mixer_selem_get_playback_volume(elem, chn, &lval);
-						snd_mixer_selem_set_playback_volume(elem, chn, get_volume_simple(&ptr, pmin, pmax, lval));
-					} else {
-						get_volume_simple(&ptr, 0, 100, 0);
-					}
+					if (set_volume_simple(elem, chn, &ptr, 0) >= 0)
+						check_flag = 1;
 				} else if (simple_skip_word(&ptr, "cap") || simple_skip_word(&ptr, "rec") ||
 					   simple_skip_word(&ptr, "nocap") || simple_skip_word(&ptr, "norec")) {
 					/* nothing */
@@ -1409,29 +1502,30 @@ static int sset(unsigned int argc, char *argv[], int roflag, int keep_handle)
 				sptr = ptr;
 				if (!strncmp(ptr, "cap", 3) && snd_mixer_selem_has_capture_switch(elem)) {
 					snd_mixer_selem_get_capture_switch(elem, chn, &ival);
-					snd_mixer_selem_set_capture_switch(elem, chn, get_bool_simple(&ptr, "cap", 0, ival));
+					if (snd_mixer_selem_set_capture_switch(elem, chn, get_bool_simple(&ptr, "cap", 0, ival)) >= 0)
+						check_flag = 1;
 				} else if (!strncmp(ptr, "rec", 3) && snd_mixer_selem_has_capture_switch(elem)) {
 					snd_mixer_selem_get_capture_switch(elem, chn, &ival);
-					snd_mixer_selem_set_capture_switch(elem, chn, get_bool_simple(&ptr, "rec", 0, ival));
+					if (snd_mixer_selem_set_capture_switch(elem, chn, get_bool_simple(&ptr, "rec", 0, ival)) >= 0)
+						check_flag = 1;
 				} else if (!strncmp(ptr, "nocap", 5) && snd_mixer_selem_has_capture_switch(elem)) {
 					snd_mixer_selem_get_capture_switch(elem, chn, &ival);
-					snd_mixer_selem_set_capture_switch(elem, chn, get_bool_simple(&ptr, "nocap", 1, ival));
+					if (snd_mixer_selem_set_capture_switch(elem, chn, get_bool_simple(&ptr, "nocap", 1, ival)) >= 0)
+						check_flag = 1;
 				} else if (!strncmp(ptr, "norec", 5) && snd_mixer_selem_has_capture_switch(elem)) {
 					snd_mixer_selem_get_capture_switch(elem, chn, &ival);
-					snd_mixer_selem_set_capture_switch(elem, chn, get_bool_simple(&ptr, "norec", 1, ival));
+					if (snd_mixer_selem_set_capture_switch(elem, chn, get_bool_simple(&ptr, "norec", 1, ival)) >= 0)
+						check_flag = 1;
 				} else if (!strncmp(ptr, "toggle", 6) && snd_mixer_selem_has_capture_switch(elem)) {
 					if (firstchn || !snd_mixer_selem_has_capture_switch_joined(elem)) {
 						snd_mixer_selem_get_capture_switch(elem, chn, &ival);
-						snd_mixer_selem_set_capture_switch(elem, chn, (ival ? 1 : 0) ^ 1);
+						if (snd_mixer_selem_set_capture_switch(elem, chn, (ival ? 1 : 0) ^ 1) >= 0)
+							check_flag = 1;
 					}
 					simple_skip_word(&ptr, "toggle");
 				} else if (isdigit(*ptr) || *ptr == '-' || *ptr == '+') {
-					if (snd_mixer_selem_has_capture_volume(elem)) {
-						snd_mixer_selem_get_capture_volume(elem, chn, &lval);
-						snd_mixer_selem_set_capture_volume(elem, chn, get_volume_simple(&ptr, cmin, cmax, lval));
-					} else {
-						get_volume_simple(&ptr, 0, 100, 0);
-					}
+					if (set_volume_simple(elem, chn, &ptr, 1) >= 0)
+						check_flag = 1;
 				} else if (simple_skip_word(&ptr, "mute") || simple_skip_word(&ptr, "off") ||
 					   simple_skip_word(&ptr, "unmute") || simple_skip_word(&ptr, "on")) {
 					/* nothing */
@@ -1442,9 +1536,9 @@ static int sset(unsigned int argc, char *argv[], int roflag, int keep_handle)
 			if (okflag == 0) {
 				if (debugflag) {
 					if (dir & 1)
-						error("Unknown playback setup '%s'..\n", ptr);
+						error("Unknown playback setup '%s'..", ptr);
 					if (dir & 2)
-						error("Unknown capture setup '%s'..\n", ptr);
+						error("Unknown capture setup '%s'..", ptr);
 				}
 				if (! keep_handle) {
 					snd_mixer_close(handle);
@@ -1456,7 +1550,15 @@ static int sset(unsigned int argc, char *argv[], int roflag, int keep_handle)
 				ptr = optr;
 			firstchn = 0;
 		}
-	} 
+	}
+	if (!check_flag) {
+		error("Invalid command!");
+		if (! keep_handle) {
+			snd_mixer_close(handle);
+			handle = NULL;
+		}
+		return 1;
+	}
       __skip_write:
 	if (!quiet) {
 		printf("Simple mixer control '%s',%i\n", snd_mixer_selem_id_get_name(sid), snd_mixer_selem_id_get_index(sid));
@@ -1758,7 +1860,7 @@ int main(int argc, char *argv[])
 				if (i >= 0 && i < 32)
 					sprintf(card, "hw:%i", i);
 				else {
-					fprintf(stderr, "\07Invalid card number.\n");
+					fprintf(stderr, "Invalid card number.\n");
 					morehelp++;
 				}
 			}
@@ -1799,7 +1901,7 @@ int main(int argc, char *argv[])
 			read_stdin = 1;
 			break;
 		default:
-			fprintf(stderr, "\07Invalid switch or option needs an argument.\n");
+			fprintf(stderr, "Invalid switch or option needs an argument.\n");
 			morehelp++;
 		}
 	}
