@@ -116,7 +116,6 @@ static void begin_voc(int fd, size_t count);
 static void end_voc(int fd);
 static void begin_wave(int fd, size_t count);
 static void end_wave(int fd);
-static void end_raw(int fd);
 static void begin_au(int fd, size_t count);
 static void end_au(int fd);
 
@@ -126,9 +125,10 @@ struct fmt_capture {
 	char *what;
 	long long max_filesize;
 } fmt_rec_table[] = {
-	{	NULL,		end_raw,	N_("raw data"),		LLONG_MAX },
-	{	begin_voc,	end_voc,	N_("VOC"),		16000000 },
-	{	begin_wave,	end_wave,	N_("WAVE"),		2000000000 },
+	{	NULL,		NULL,		N_("raw data"),		LLONG_MAX },
+	{	begin_voc,	end_voc,	N_("VOC"),		16000000LL },
+	/* FIXME: can WAV handle exactly 2GB or less than it? */
+	{	begin_wave,	end_wave,	N_("WAVE"),		2147483648LL },
 	{	begin_au,	end_au,		N_("Sparc Audio"),	LLONG_MAX }
 };
 
@@ -155,7 +155,7 @@ _("Usage: %s [OPTION]... [FILE]...\n"
 "-h, --help              help\n"
 "    --version           print current version\n"
 "-l, --list-devices      list all soundcards and digital audio devices\n"
-"-L, --list-pcms         list all PCMs defined\n"
+"-L, --list-pcms         list device names\n"
 "-D, --device=NAME       select PCM by name\n"
 "-q, --quiet             quiet mode\n"
 "-t, --file-type TYPE    file type (voc, wav, raw or au)\n"
@@ -190,24 +190,6 @@ _("Usage: %s [OPTION]... [FILE]...\n"
 	printf(_("-f dat (16 bit little endian, 48000, stereo)\n"));
 }
 
-static void names_list(void)
-{
-	int err;
-	snd_devname_t *list, *item;
-
-	err = snd_names_list("pcm", &list);
-	if (err < 0) {
-		error(_("snd_names_list error: %s"), snd_strerror(err));
-		return;
-	}
-	item = list;
-	while (item) {
-		printf("%s [%s]\n", item->name, item->comment);
-		item = item->next;
-	}
-	snd_names_list_free(list);
-}
-
 static void device_list(void)
 {
 	snd_ctl_t *handle;
@@ -280,21 +262,42 @@ static void device_list(void)
 
 static void pcm_list(void)
 {
-	snd_config_t *conf;
-	snd_output_t *out;
-	int err = snd_config_update();
-	if (err < 0) {
-		error("snd_config_update: %s", snd_strerror(err));
+	void **hints, **n;
+	char *name, *descr, *descr1, *io;
+	const char *filter;
+
+	if (snd_device_name_hint(-1, "pcm", &hints) < 0)
 		return;
+	n = hints;
+	filter = stream == SND_PCM_STREAM_CAPTURE ? "Input" : "Output";
+	while (*n != NULL) {
+		name = snd_device_name_get_hint(*n, "NAME");
+		descr = snd_device_name_get_hint(*n, "DESC");
+		io = snd_device_name_get_hint(*n, "IOID");
+		if (io != NULL && strcmp(io, filter) == 0)
+			goto __end;
+		printf("%s\n", name);
+		if ((descr1 = descr) != NULL) {
+			printf("    ");
+			while (*descr1) {
+				if (*descr1 == '\n')
+					printf("\n    ");
+				else
+					putchar(*descr1);
+				descr1++;
+			}
+			putchar('\n');
+		}
+	      __end:
+	      	if (name != NULL)
+	      		free(name);
+		if (descr != NULL)
+			free(descr);
+		if (io != NULL)
+			free(io);
+		n++;
 	}
-	err = snd_output_stdio_attach(&out, stdout, 0);
-	assert(err >= 0);
-	err = snd_config_search(snd_config, "pcm", &conf);
-	if (err < 0)
-		return;
-	printf(_("PCM list:\n"));
-	snd_config_save(conf, out);
-	snd_output_close(out);
+	snd_device_name_free_hint(hints);
 }
 
 static void version(void)
@@ -309,7 +312,10 @@ static void signal_handler(int sig)
 	if (!quiet_mode)
 		fprintf(stderr, _("Aborted by signal %s...\n"), strsignal(sig));
 	if (stream == SND_PCM_STREAM_CAPTURE) {
-		fmt_rec_table[file_type].end(fd);
+		if (fmt_rec_table[file_type].end) {
+			fmt_rec_table[file_type].end(fd);
+			fd = -1;
+		}
 		stream = -1;
 	}
 	if (fd > 1) {
@@ -364,7 +370,7 @@ int main(int argc, char *argv[])
 	};
 	char *pcm_name = "default";
 	int tmp, err, c;
-	int do_names_list = 0, do_device_list = 0, do_pcm_list = 0;
+	int do_device_list = 0, do_pcm_list = 0;
 	snd_pcm_info_t *info;
 
 #ifdef ENABLE_NLS
@@ -405,9 +411,6 @@ int main(int argc, char *argv[])
 		case OPT_VERSION:
 			version();
 			return 0;
-		case 'n':
-			do_names_list = 1;
-			break;
 		case 'l':
 			do_device_list = 1;
 			break;
@@ -528,18 +531,13 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	if (do_names_list) {
-		names_list();
-		return 0;
-	} else if (do_device_list) {
+	if (do_device_list) {
 		if (do_pcm_list) pcm_list();
 		device_list();
-		snd_config_update_free_global();
-		return 0;
+		goto __end;
 	} else if (do_pcm_list) {
 		pcm_list();
-		snd_config_update_free_global();
-		return 0;
+		goto __end;
 	}
 
 	err = snd_pcm_open(&handle, pcm_name, stream, open_mode);
@@ -610,6 +608,7 @@ int main(int argc, char *argv[])
 		putchar('\n');
 	snd_pcm_close(handle);
 	free(audiobuf);
+      __end:
 	snd_output_close(log);
 	snd_config_update_free_global();
 	return EXIT_SUCCESS;
@@ -917,8 +916,9 @@ static void set_params(void)
 	if ((float)rate * 1.05 < hwparams.rate || (float)rate * 0.95 > hwparams.rate) {
 		if (!quiet_mode) {
 			char plugex[64];
+			const char *pcmname = snd_pcm_name(handle);
 			fprintf(stderr, _("Warning: rate is not accurate (requested = %iHz, got = %iHz)\n"), rate, hwparams.rate);
-			if (strchr(snd_pcm_name(handle), ':'))
+			if (! pcmname || strchr(snd_pcm_name(handle), ':'))
 				*plugex = 0;
 			else
 				snprintf(plugex, sizeof(plugex), "(-Dplug:%s)",
@@ -1235,6 +1235,7 @@ static void compute_max_peak(u_char *data, size_t count)
 			else
 				putchar(' ');
 		printf(" %i%%\n", perc);
+		fflush(stdout);
 	}
 }
 
@@ -1882,12 +1883,6 @@ static void end_voc(int fd)
 		close(fd);
 }
 
-static void end_raw(int fd)
-{                              /* REALLY only close output */
-       if (fd != 1)
-               close(fd);
-}
-
 static void end_wave(int fd)
 {				/* only close output */
 	WaveChunkHeader cd;
@@ -1926,6 +1921,8 @@ static void end_au(int fd)
 static void header(int rtype, char *name)
 {
 	if (!quiet_mode) {
+		if (! name)
+			name = (stream == SND_PCM_STREAM_PLAYBACK) ? "stdout" : "stdin";
 		fprintf(stderr, "%s %s '%s' : ",
 			(stream == SND_PCM_STREAM_PLAYBACK) ? _("Playing") : _("Recording"),
 			gettext(fmt_rec_table[rtype].what),
@@ -2053,26 +2050,62 @@ static void playback(char *name)
 		close(fd);
 }
 
-static void capture(char *name_)
+static int new_capture_file(char *name, char *namebuf, size_t namelen,
+			    int filecount)
+{
+	/* get a copy of the original filename */
+	char *s;
+	char buf[PATH_MAX+1];
+
+	strncpy(buf, name, sizeof(buf));
+
+	/* separate extension from filename */
+	s = buf + strlen(buf);
+	while (s > buf && *s != '.' && *s != '/')
+		--s;
+	if (*s == '.')
+		*s++ = 0;
+	else if (*s == '/')
+		s = buf + strlen(buf);
+
+	/* upon first jump to this if block rename the first file */
+	if (filecount == 1) {
+		if (*s)
+			snprintf(namebuf, namelen, "%s-01.%s", buf, s);
+		else
+			snprintf(namebuf, namelen, "%s-01", buf);
+		remove(namebuf);
+		rename(name, namebuf);
+		filecount = 2;
+	}
+
+	/* name of the current file */
+	if (*s)
+		snprintf(namebuf, namelen, "%s-%02i.%s", buf, filecount, s);
+	else
+		snprintf(namebuf, namelen, "%s-%02i", buf, filecount);
+
+	return filecount;
+}
+
+static void capture(char *orig_name)
 {
 	int tostdout=0;		/* boolean which describes output stream */
 	int filecount=0;	/* number of files written */
-	char *name=name_;	/* current filename */
+	char *name = orig_name;	/* current filename */
 	char namebuf[PATH_MAX+1];
-	off64_t cur;		/* number of bytes to capture */
+	off64_t count, rest;		/* number of bytes to capture */
 
 	/* get number of bytes to capture */
-	pbrec_count = calc_count();
+	count = calc_count();
+	if (count == 0)
+		count = LLONG_MAX;
 	/* WAVE-file should be even (I'm not sure), but wasting one byte
 	   isn't a problem (this can only be in 8 bit mono) */
-	if (pbrec_count < LLONG_MAX)
-		pbrec_count += pbrec_count % 2;
+	if (count < LLONG_MAX)
+		count += count % 2;
 	else
-		pbrec_count -= pbrec_count % 2;
-	if (pbrec_count == 0)
-		pbrec_count -= 2;
-
-	cur = pbrec_count;
+		count -= count % 2;
 
 	/* display verbose output to console */
 	header(file_type, name);
@@ -2085,93 +2118,65 @@ static void capture(char *name_)
 		fd = fileno(stdout);
 		name = "stdout";
 		tostdout=1;
-		fdcount = 0;
+		if (count > fmt_rec_table[file_type].max_filesize)
+			count = fmt_rec_table[file_type].max_filesize;
 	}
 
 	do {
 		/* open a file to write */
 		if(!tostdout) {
-
 			/* upon the second file we start the numbering scheme */
-			if(filecount)
-			{
-				/* get a copy of the original filename */
-				char *s;
-				char buf[PATH_MAX+1];
-				strncpy(buf, name_, sizeof(buf));
-
-				/* separate extension from filename */
-				s=buf+strlen(buf);
-				while(s>buf && *s!='.' && *s!='/')
-					--s;
-				if(*s=='.')
-					*s++=0;
-				else if(*s=='/')
-					s=buf+strlen(buf);
-
-				/* upon first jump to this if block rename the first file */
-				if(filecount==1) {
-					if(*s)
-						snprintf(namebuf, sizeof(namebuf), "%s-01.%s", buf, s);
-					else
-						snprintf(namebuf, sizeof(namebuf), "%s-01", buf);
-					remove(namebuf);
-					rename(name, namebuf);
-					filecount=2;
-				}
-
-				/* name of the current file */
-				if(*s)
-					snprintf(namebuf, sizeof(namebuf), "%s-%02i.%s", buf, filecount, s);
-				else
-					snprintf(namebuf, sizeof(namebuf), "%s-%02i", buf, filecount);
-				name=namebuf;
-
+			if (filecount) {
+				filecount = new_capture_file(orig_name, namebuf,
+							     sizeof(namebuf),
+							     filecount);
+				name = namebuf;
 			}
-
+			
 			/* open a new file */
 			remove(name);
 			if ((fd = open64(name, O_WRONLY | O_CREAT, 0644)) == -1) {
 				perror(name);
 				exit(EXIT_FAILURE);
 			}
-			fdcount = 0;
 			filecount++;
 		}
 
+		rest = count;
+		if (rest > fmt_rec_table[file_type].max_filesize)
+			rest = fmt_rec_table[file_type].max_filesize;
+
 		/* setup sample header */
 		if (fmt_rec_table[file_type].start)
-			fmt_rec_table[file_type].start(fd, pbrec_count);
+			fmt_rec_table[file_type].start(fd, rest);
 
 		/* capture */
-		do {
-			ssize_t r=0;
-			for (; cur > 0 && fdcount<fmt_rec_table[file_type].max_filesize; cur -= r) {
-				ssize_t err;
-				size_t c = (cur <= chunk_bytes) ? cur : chunk_bytes;
-				c = c * 8 / bits_per_frame;
-				if ((size_t)(r = pcm_read(audiobuf, c)) != c)
-					break;
-				r = r * bits_per_frame / 8;
-				if ((err = write(fd, audiobuf, r)) != r) {
-					perror(name);
-					exit(EXIT_FAILURE);
-				}
-				if (err > 0)
-					fdcount += err;
+		fdcount = 0;
+		while (rest > 0) {
+			size_t c = (rest <= (off64_t)chunk_bytes) ?
+				(size_t)rest : chunk_bytes;
+			size_t f = c * 8 / bits_per_frame;
+			if (pcm_read(audiobuf, f) != f)
+				break;
+			if (write(fd, audiobuf, c) != c) {
+				perror(name);
+				exit(EXIT_FAILURE);
 			}
-			/* exit conditions:
-			   1) format_raw and a timelimit
-			   2) all requested samples/bytes have been captured (cur>0)
-			   3) filesize threshold was reached (fdcount<wrthreshold)
-			*/
-		} while (file_type == FORMAT_RAW && !timelimit && cur>0 && fdcount<fmt_rec_table[file_type].max_filesize);
+			count -= c;
+			rest -= c;
+			fdcount += c;
+		}
 
 		/* finish sample container */
-		fmt_rec_table[file_type].end(fd);
+		if (fmt_rec_table[file_type].end && !tostdout) {
+			fmt_rec_table[file_type].end(fd);
+			fd = -1;
+		}
 
-		/* repeat the loop when format is raw without timelimit or filesize threshold was reached */
-	} while((file_type == FORMAT_RAW && !timelimit) || fdcount>=fmt_rec_table[file_type].max_filesize);
+		/* repeat the loop when format is raw without timelimit or
+		 * requested counts of data are recorded
+		 */
+	} while ((file_type == FORMAT_RAW && !timelimit) || count > 0);
 }
 
 void playbackv_go(int* fds, unsigned int channels, size_t loaded, off64_t count, int rtype, char **names)
